#!/bin/bash
echo "Starting php profiler installer..."
EXT_PKG_DIR="$(pwd)"

echo "Setting Package Directory: ${EXT_PKG_DIR}"

SUPPORTED_VERSIONS=("5.5" "5.6" "7.0" "7.1" "7.2" "7.3" "7.4" "8.0" "8.1" "8.2")

PHP_DIR_LIST=(
  "/usr/local"
  "/opt" # xampp under for linux
  "/usr/bin"
  "/Applications/XAMPP/" # osx
)

phpbinaries=()
phpresults=()

##############################
# Checks if system is 64 bit
# Returns:
#   1/0 (false/true)
##############################
is_64bit() {
    # this will check if the machine is 64bit and if so return true
    if [[ "`uname -m`" == "x86_64" ]]; then
        return 0
    else
        return 1
    fi
}

##############################
# Checks if system is arm
# Returns:
#   1/0 (false/true)
##############################
is_arm() {
    local uname_result=$(uname -m)
    if [[ ${uname_result} == *"arm"*  || ${uname_result} == *"aarch"* ]]; then
        return 0
    else
        return 1
    fi
}

is_32bit() {
    local bit_result=$(getconf LONG_BIT)
    if [[ ${bit_result} == "32" ]]; then
        return 0
    else
        return 1
    fi
}

is_supported_version () {
  # $1 PHP binary
  for ver in ${SUPPORTED_VERSIONS[@]}; do
    if [[ $1 = $ver ]]; then
      echo 0
      return
    fi
  done
  echo 1
}

phpbinaries=()
phpresults=()

if [[ $# -ge 1 ]]; then
  PHP_DIR_LIST=()
  for DIR_OR_FILE in "$@"; do
      if [[ ! -z "$DIR_OR_FILE" ]]; then
        PHP_BIN_INPUT_FILE_OR_DIR=$DIR_OR_FILE
        if [[ -d "$PHP_BIN_INPUT_FILE_OR_DIR" ]]; then
          PHP_DIR_LIST+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom folder set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi

        if [[ -f "$PHP_BIN_INPUT_FILE_OR_DIR" ]] && [[ -x $PHP_BIN_INPUT_FILE_OR_DIR ]]; then
          phpbinaries+=($PHP_BIN_INPUT_FILE_OR_DIR)
          echo "Custom binary set: $PHP_BIN_INPUT_FILE_OR_DIR"
        fi
      fi
  done
fi

for php_dir in ${PHP_DIR_LIST[@]}; do
  if [[ -d "$php_dir" ]]; then
    echo "Checking folder at $php_dir"
    phpresults+=($(find -L "$php_dir" -type f -iregex ".*\/php[@]*[0-9]*[.]*[0-9]*" 2> /dev/null))
  fi
done

# glob php versions in files
for PHP_RES in ${phpresults[@]}; do
	# check if we support it
  if [[ -f $PHP_RES ]] && [[ -x $PHP_RES ]]; then
    if [[ $(is_supported_version $($PHP_RES -n -r "echo PHP_MAJOR_VERSION.'.'.PHP_MINOR_VERSION;")) ]]; then
      # a dd to list of php binaries
      if [[ -L $PHP_RES ]]; then
        PHP_RES_LINK=$(readlink $PHP_RES)
        if [[ -f $PHP_RES_LINK ]] && [[ -x $PHP_RES_LINK ]]; then
          echo "Found binary (link): $PHP_RES_LINK"
          phpbinaries+=($PHP_RES_LINK)
        fi
      else
        echo "Found binary: $PHP_RES"
        phpbinaries+=($PHP_RES)
      fi
    else
      echo "Not found binary: $PHP_RES"
    fi
  fi
done

printf "Removing duplicates...\n\n"
deduped=($(echo ${phpbinaries[*]} | tr ' ' '\n' | sort -u))
if [[ ${#deduped[*]} -eq 0 ]]; then
     echo "Unable to find PHP installation."
     exit 1
fi

# update permissions
chmod 664 ./*.so > /dev/null 2>&1
chmod 664 ./*.php > /dev/null 2>&1
chmod 775 ./linux_* > /dev/null 2>&1
chmod 775 ./linux_*/*.so > /dev/null 2>&1

for phpbin in ${deduped[@]}; do
    echo "Using binary: $phpbin"
    if hash $phpbin 2>/dev/null; then
        CURRENT_DIR=$EXT_PKG_DIR
        EXTENSION_DIR=$($phpbin -r 'echo "[".ini_get("extension_dir")."]";')
        EXTENSION_DIR=${EXTENSION_DIR/#*\[}
        EXTENSION_DIR=${EXTENSION_DIR/%\]}
        PHP_MAJOR_VERSION=`$phpbin -n -r 'echo PHP_MAJOR_VERSION;'`
        PHP_MINOR_VERSION=`$phpbin -n -r 'echo PHP_MINOR_VERSION;'`
        PHP_ZTS_ENABLED=`$phpbin -n -r 'echo PHP_ZTS;'`
        PHP_DEBUG_ENABLED=`$phpbin -n -r 'echo PHP_DEBUG;'`

        PHP_VERSION="${PHP_MAJOR_VERSION}.${PHP_MINOR_VERSION}";
        if [[ "$PHP_ZTS_ENABLED" == "1" ]]; then
            PHP_VERSION="${PHP_VERSION}.zts";
        fi

        if [[ ! -d "$EXTENSION_DIR" ]]; then
          echo "Unable to find extension location: ${PHP_VERSION}\n\n"
          continue
        fi

        echo "Detected PHP ${PHP_VERSION}" 
        echo "Found extension location: ${EXTENSION_DIR}"

        if (is_arm); then
            EXTENSION_FILE="${CURRENT_DIR}/linux_arm64v8/stackify-${PHP_VERSION}.so"
            if (is_32bit); then
                EXTENSION_FILE="${CURRENT_DIR}/linux_arm32v7/stackify-${PHP_VERSION}.so"
            fi
        else
            EXTENSION_FILE="${CURRENT_DIR}/linux_32/stackify-${PHP_VERSION}.so"
            if (is_64bit); then
                EXTENSION_FILE="${CURRENT_DIR}/linux_64/stackify-${PHP_VERSION}.so"
            fi
        fi

        if [[ -f "$EXTENSION_FILE" ]]; then
            echo "Using package extension: ${EXTENSION_FILE}"
            echo "Using package wrapper: ${CURRENT_DIR}/Stackify.php"

            echo "Installing into $EXTENSION_DIR"
            unlink ${EXTENSION_DIR}/stackify.so > /dev/null 2>&1
            unlink ${EXTENSION_DIR}/Stackify.php > /dev/null 2>&1
            ln -s  ${EXTENSION_FILE} ${EXTENSION_DIR}/stackify.so > /dev/null 2>&1
            ln -s  ${CURRENT_DIR}/Stackify.php ${EXTENSION_DIR}/Stackify.php > /dev/null 2>&1
        else
          echo "Stackify extension is not found. PHP Version not supported."
          echo "Extension file: ${EXTENSION_FILE}"
          echo "Unable to install extension."
        fi

        if [[ ! -f "${CURRENT_DIR}/stackify.ini" ]]; then
          continue
        fi

        # Install INI file
        PHP_CONFIG_FILE_SCAN_DIR=$($phpbin -r 'echo "[".PHP_CONFIG_FILE_SCAN_DIR."]";')
        PHP_CONFIG_FILE_SCAN_DIR=${PHP_CONFIG_FILE_SCAN_DIR/#*\[}
        PHP_CONFIG_FILE_SCAN_DIR=${PHP_CONFIG_FILE_SCAN_DIR/%\]}

        # Where PHP INI is located
        PHP_INI_LOCATION=$($phpbin -r 'echo "[".dirname(php_ini_loaded_file())."]";')
        PHP_INI_LOCATION=${PHP_INI_LOCATION/#*\[}
        PHP_INI_LOCATION=${PHP_INI_LOCATION/%\]}

        # Where PHP INI path is located
        PHP_INI_PATH_LOCATION=$($phpbin -r 'echo "[".PHP_CONFIG_FILE_PATH."]";')
        PHP_INI_PATH_LOCATION=${PHP_INI_LOCATION/#*\[}
        PHP_INI_PATH_LOCATION=${PHP_INI_LOCATION/%\]}

        echo "Setting PHP INI path: ${PHP_INI_PATH_LOCATION}"
        if [[ -n $PHP_CONFIG_FILE_SCAN_DIR && -d $PHP_CONFIG_FILE_SCAN_DIR ]]; then
            if [[ ! -d ${PHP_CONFIG_FILE_SCAN_DIR} ]]; then
              mkdir -p ${PHP_CONFIG_FILE_SCAN_DIR}
            fi

            # If we have the config file, ignore
            if [[ ! -f "${PHP_CONFIG_FILE_SCAN_DIR}/99-stackify.ini"  && ! -f "${PHP_CONFIG_FILE_SCAN_DIR}/stackify.ini" ]]; then
              # if config file scan directory is not null, install .ini there
              cp "${CURRENT_DIR}/stackify.ini" "${PHP_CONFIG_FILE_SCAN_DIR}/99-stackify.ini"
              echo "Added .ini in PHP_CONFIG_FILE_SCAN_DIR: ${PHP_CONFIG_FILE_SCAN_DIR}/99-stackify.ini"
            else
              echo "File .ini already exists in PHP_CONFIG_FILE_SCAN_DIR: ${PHP_CONFIG_FILE_SCAN_DIR}"
            fi

            # Check relative path - this works for ubuntu
            echo "Setting PHP CLI config file scan path: ${PHP_CONFIG_FILE_SCAN_DIR}"
            for CONFIG_SCAN_DIR_PREFIX in /etc/php5* /etc/php7 /etc/php8 /etc/php/[578].*; do
              if [[ "${PHP_CONFIG_FILE_SCAN_DIR}" = "${CONFIG_SCAN_DIR_PREFIX}/cli/conf.d" ]]; then
                echo "Matching directory: ${CONFIG_SCAN_DIR_PREFIX}"

                # Check fpm and apache
                CONFIG_SCAN_DIR_APACHE=$CONFIG_SCAN_DIR_PREFIX/apache2/conf.d
                if [ -d "${CONFIG_SCAN_DIR_APACHE}" ]; then
                  # If we have the config file, ignore
                  if [[ ! -f "${CONFIG_SCAN_DIR_APACHE}/99-stackify.ini" && ! -f "${CONFIG_SCAN_DIR_APACHE}/stackify.ini" ]]; then
                    # if config file scan directory is not null, install .ini there
                    cp "${CURRENT_DIR}/stackify.ini" "${CONFIG_SCAN_DIR_APACHE}/99-stackify.ini"
                    echo "Added .ini in CONFIG_SCAN_DIR_APACHE: ${CONFIG_SCAN_DIR_APACHE}/99-stackify.ini"
                  else
                    echo "File .ini already exists in CONFIG_SCAN_DIR_APACHE: ${CONFIG_SCAN_DIR_APACHE}"
                  fi
                fi

                CONFIG_SCAN_DIR_FPM=$CONFIG_SCAN_DIR_PREFIX/fpm/conf.d
                if [ -d "${CONFIG_SCAN_DIR_FPM}" ]; then
                  # If we have the config file, ignore
                  if [[ ! -f "${CONFIG_SCAN_DIR_FPM}/99-stackify.ini" && ! -f "${CONFIG_SCAN_DIR_FPM}/stackify.ini" ]]; then
                    # if config file scan directory is not null, install .ini there
                    cp "${CURRENT_DIR}/stackify.ini" "${CONFIG_SCAN_DIR_FPM}/99-stackify.ini"
                    echo "Added .ini in CONFIG_SCAN_DIR_FPM: ${CONFIG_SCAN_DIR_FPM}/99-stackify.ini"
                  else
                    echo "File .ini already exists in CONFIG_SCAN_DIR_FPM: ${CONFIG_SCAN_DIR_FPM}"
                  fi
                fi
              fi
            done
        fi
    else
        echo "Unable to find PHP installation: $phpbin"
    fi
done


echo "If you get a warning 'PHP Warning:  Module stackify is already loaded in Unknown on line 0'."
echo "Please check the 'php.ini' file for that version and remove the 'extension=stackify*' line"
echo "Check the PHP Info page if you got the right information under the Stackify extension table"
echo "Done"
